import pytest
from asphalt.core import Context
from motor.motor_asyncio import AsyncIOMotorClient

from asphalt.mongodb.component import MongoDBComponent


@pytest.mark.asyncio
async def test_default_client(caplog):
    """Test that the default connection is started and is available on the context."""
    async with Context() as context:
        await MongoDBComponent().start(context)
        assert isinstance(context.mongo, AsyncIOMotorClient)

    records = [record for record in caplog.records if record.name == 'asphalt.mongodb.component']
    records.sort(key=lambda r: r.message)
    assert len(records) == 2
    assert records[0].message == ("Configured MongoDB client (default / ctx.mongo; "
                                  "hosts={'localhost:27017'})")
    assert records[1].message == 'MongoDB client (default) shut down'


@pytest.mark.asyncio
async def test_multiple_clients(caplog):
    """Test that a multiple connection configuration works as intended."""
    async with Context() as context:
        await MongoDBComponent(clients={
            'db1': {'host': 'localhost:27018'},
            'db2': {'host': '/tmp/mongodb.sock', 'ssl': True}
        }).start(context)
        assert isinstance(context.db1, AsyncIOMotorClient)
        assert isinstance(context.db2, AsyncIOMotorClient)

    records = [record for record in caplog.records if record.name == 'asphalt.mongodb.component']
    records.sort(key=lambda r: r.message)
    assert len(records) == 4
    assert records[0].message == ("Configured MongoDB client (db1 / ctx.db1; "
                                  "hosts={'localhost:27018'})")
    assert records[1].message == ("Configured MongoDB client (db2 / ctx.db2; "
                                  "hosts={'/tmp/mongodb.sock'})")
    assert records[2].message == 'MongoDB client (db1) shut down'
    assert records[3].message == 'MongoDB client (db2) shut down'


@pytest.mark.asyncio
async def test_create_remove_document():
    """Test the client against a real MongoDB server."""
    async with Context() as context:
        await MongoDBComponent().start(context)
        collection = context.mongo['testdb']['testcollection']
        document = {'somekey': 'somevalue'}
        document2 = {'otherkey': 7}
        try:
            await collection.insert(document)
            await collection.insert(document2)
            documents = await collection.find().to_list(None)
            assert documents == [document, document2]
        finally:
            await context.mongo.drop_database('testdb')



import logging
from typing import Dict, Any

from async_generator import yield_
from motor.motor_asyncio import AsyncIOMotorClient
from typeguard import check_argument_types

from asphalt.core import Component, Context, merge_config, context_teardown

logger = logging.getLogger(__name__)


class MongoDBComponent(Component):
    """
    Creates one or more :class:`~motor.motor_asyncio.AsyncIOMotorClient` resources.

    If ``clients`` is given, a MongoDB client resource will be added for each key in the
    dictionary, using the key as the resource name. Any extra keyword arguments to the component
    constructor will be used as defaults for omitted configuration values. The context attribute
    will by default be the same as the resource name, unless explicitly set with the
    ``context_attr`` option.

    If ``clients`` is omitted, a single MongoDB client resource (``default`` / ``ctx.mongo``)
    is added using any extra keyword arguments passed to the component.

    The client(s) will not connect to the target database until they're used for the first time.

    :param clients: a dictionary of resource name ⭢
        :class:`~motor.motor_asyncio.AsyncIOMotorClient` arguments
    :param default_client_args: default values for omitted
        :class:`~motor.motor_asyncio.AsyncIOMotorClient` arguments
    """

    def __init__(self, clients: Dict[str, Dict[str, Any]] = None, **default_client_args):
        assert check_argument_types()
        if not clients:
            default_client_args.setdefault('context_attr', 'mongo')
            clients = {'default': default_client_args}

        self.clients = []
        for resource_name, config in clients.items():
            config = merge_config(default_client_args, config)
            context_attr = config.pop('context_attr', resource_name)
            client = AsyncIOMotorClient(**config)
            self.clients.append((resource_name, context_attr, client))

    @context_teardown
    async def start(self, ctx: Context):
        for resource_name, context_attr, client in self.clients:
            ctx.add_resource(client, resource_name, context_attr)
            hosts = {'%s:%d' % (host, port) if port is not None else host
                     for host, port in client.delegate._topology_settings.seeds}
            logger.info('Configured MongoDB client (%s / ctx.%s; hosts=%r)', resource_name,
                        context_attr, hosts)

        await yield_()

        for resource_name, context_attr, client in self.clients:
            client.close()
            logger.info('MongoDB client (%s) shut down', resource_name)
